Package: dir.expiry
Version: 1.18.0
Date: 2024-10-17
Title: Managing Expiration for Cache Directories
Description: Implements an expiration system for access to versioned directories.
    Directories that have not been accessed by a registered function within a certain time frame are deleted.
    This aims to reduce disk usage by eliminating obsolete caches generated by old versions of packages.
Authors@R: person("Aaron", "Lun", role=c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
License: GPL-3
Imports: utils, filelock
Suggests: rmarkdown, knitr, testthat, BiocStyle
biocViews: Software, Infrastructure
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/dir.expiry
git_branch: RELEASE_3_22
git_last_commit: 6d768b3
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:37:50 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
