# Generated by roxygen2: do not edit by hand

export(clearDirectories)
export(lockDirectory)
export(touchDirectory)
export(unlockDirectory)
importFrom(filelock,lock)
importFrom(filelock,unlock)
importFrom(utils,packageVersion)
